from .utils import (
    map_none,
    extract_href_or_text,
    extract_img_or_text,
    get_content_by_title,
)
import markdownify
import html2text


def parse_genres(tag):
    genres = {}
    for link in tag.find_all("a"):
        genres[link.get_text()] = int(
            link.next_sibling.get_text().replace("(", "").replace(")", "")
        )
    return genres


def parse_author(soup, desc_format="plain"):
    author = {}
    author["header_name"] = soup.find(
        "span", class_=["releasestitle", "tabletitle"]
    ).get_text()
    # content that is just either text or a link
    normal_content = [
        ("name", "Name (in native language)"),
        ("birth_place", "Birth Place"),
        ("birth_date", "Birth Date"),
        ("zodiac", "Zodiac"),
        ("last_updated", "Last Updated"),
        ("blood_type", "Blood Type"),
        ("gender", "Gender"),
        ("website", "Official Website"),
        ("twitter", "Twitter"),
        ("facebook", "Facebook"),
    ]
    for (key, title) in normal_content:
        author[key] = map_none(get_content_by_title(soup, title), extract_href_or_text)

    author["image"] = map_none(get_content_by_title(soup, "Image"), extract_img_or_text)
    genres = get_content_by_title(soup, "Genres")
    if genres:
        author["genres"] = parse_genres(genres)
    else:
        author["genres"] = None

    desc = get_content_by_title(soup, "Comments")
    try:
        if desc_format == "markdown":
            desc = markdownify(str(desc))
        elif desc_format == "raw":
            desc = str(desc)
        elif desc_format == "plain":
            text_converter = html2text.HTML2Text()
            text_converter.ignore_links = True
            text_converter.unicode_snob = True
            text_converter.ignore_emphasis = True
            text_converter.ignore_anchors = True
            text_converter.ignore_images = True
            desc = text_converter.handle(str(desc))
        else:
            desc = desc.get_text()
    except IndexError:
        desc = ""

    author["comments"] = desc
    author["associated_names"] = [
        markdownify.markdownify(name)
        for name in get_content_by_title(soup, "Associated Names")
        .encode_contents()
        .decode("utf-8")
        .replace("\n", "")
        .replace("</br>", "")
        .split("<br>")
        if len(name) > 0 and name != "N/A"
    ]

    # find the "Series Title (Click for series info)" div as a marker to navigate the rest of the series table. cut out the headers and the footer note
    series_table = (
        soup.find("a", string="Series Title")
        .find_parent("div")
        .find_next_siblings("div")[2:-1]
    )
    author_series = []
    i = 0
    while i < len(series_table):
        series = {}
        series["title"] = series_table[i].get_text()
        series_link = series_table[i].find("a")
        if series_link:
            series["id"] = series_link["href"].replace(
                "https://www.mangaupdates.com/series.html?id=", ""
            )
        else:
            series["id"] = None

        series["genres"] = [
            text.strip() for text in series_table[i + 1].get_text().split(",")
        ]
        series["year"] = series_table[i + 2].get_text()
        author_series.append(series)
        i += 3

    author["series"] = author_series

    return author



import re


def extract_href_or_text(content):
    if content.find("a") is not None:
        return content.find("a").get("href")
    else:
        return content.get_text()


def extract_img_or_text(content):
    if content.find("img") is not None:
        return content.find("img").get("src")
    else:
        return content.get_text()


def parse_count(text):
    result = dict()
    categories = text.split(",")
    for cat in categories:
        if len(cat) == 0:
            continue
        parsed = re.search("([^()]+)\(([^()]+)\)", cat)
        name = parsed.group(1).strip()
        count = parsed.group(2).strip()
        result[name] = int(count)
    return result


def map_none(val, f):
    return f(val) if val is not None else None


def get_content_by_title(tag, title):
    return tag.find("div", class_="sCat", string=title).find_next(
        "div", class_="sContent"
    )



from markdownify import markdownify
import re
from .utils import extract_href_or_text, parse_count


def parse_group(contents, note_format="markdown"):
    """
    Parse group info from mangaupdates.

    Parameters
    ----------
    content : BeautifulSoup
        BeautifulSoup object of the group page content.
    note_format : str, optional
        Format to transform the note into. can be 'plain', 'raw' or 'markdown'. defaults to 'markdown'.

    Returns
    -------
    series : dict
        Series information.
        ::

            {
                'name': 'Group name',
                'IRC': 'IRC link',
                'website': 'link',
                'forum': 'forum link, usually mangadex',
                'discord': 'link to discord server',
                'twitter': 'link to twitter',
                'facebook': 'link to facebook',
                'release_frequency': 'release frequency',
                'number_of_release': 'number of scanlated releases',
                'active': 'the status of the group',
                'total_series' : 'number of scanlated series',
                'genres': 'genre count of scanlated series',
                'categories': 'category count of scanlated series',
                'notes': 'useful information, like previous name'
                'series': [
                    {
                        'id': 'series id',
                        'name': 'series name',
                    }
                ]
                'releases': [
                    # TODO
                ]
            }

    """
    info = contents[0]
    # TODO
    releases = contents[1]
    series = contents[2]

    # info
    info_tags = info.find("div", class_="row").find_all(
        "div", class_="col-6", recursive=False
    )
    try:
        if note_format == "markdown":
            notes = markdownify(str(info_tags[27]))
        elif note_format == "raw":
            notes = str(info_tags[27])
        else:
            notes = info_tags[27].get_text()
    except IndexError:
        notes = ""

    group = dict(
        name=info_tags[1].get_text(),
        IRC=extract_href_or_text(info_tags[3]),
        website=extract_href_or_text(info_tags[5]),
        forum=extract_href_or_text(info_tags[7]),
        discord=extract_href_or_text(info_tags[9]),
        twitter=extract_href_or_text(info_tags[11]),
        facebook=extract_href_or_text(info_tags[13]),
        release_frequency=info_tags[15].get_text(),
        number_of_release=info_tags[17].get_text(),
        active=info_tags[19].get_text(),
        total_series=info_tags[21].get_text(),
        genres=parse_count(info_tags[23].get_text()),
        categories=parse_count(info_tags[25].get_text()),
        notes=notes,
    )

    series_parsed = []
    for s in series.find_all("div", class_="col-6"):
        if s.find("a") is None:
            continue
        id = (
            s.find("a")
            .get("href")
            .replace("https://www.mangaupdates.com/series.html?id=", "")
        )
        name = s.get_text()
        series_parsed.append(dict(id=id, name=name))
    group["series"] = series_parsed

    group["releases"] = []

    return group
